package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.icloud.mdm.sdk.base.IBaseMdmBusiExtdCloud;
import cn.estsh.i3plus.icloud.mdm.sdk.base.IBaseMdmBusiModelCloud;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mdm.bean.base.BaseMdmBusiModelBean;
import cn.estsh.i3plus.pojo.mdm.bean.base.MdmSyncData;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;

import java.util.List;

/**
 * @Description : 业务同步setvice
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-05-15 17:58
 * @Modify:
 **/
public abstract class BaseBusiSyncMdmDataService<T, M extends BaseMdmBusiModelBean> {

    private IBaseMdmBusiExtdCloud baseMdmBusiExtdCloud;

    private IBaseMdmBusiModelCloud<M> baseMdmBusiModelCloud;

    public BaseBusiSyncMdmDataService(IBaseMdmBusiExtdCloud baseMdmBusiExtdCloud, IBaseMdmBusiModelCloud<M> baseMdmBusiModelCloud) {
        this.baseMdmBusiExtdCloud = baseMdmBusiExtdCloud;
        this.baseMdmBusiModelCloud = baseMdmBusiModelCloud;
    }

    /**
     * 主数据新增同步
     * @param syncData
     */
    public void masterInsertSync(MdmSyncData syncData) {
        baseMdmBusiExtdCloud.whiteDoProcessMissBusiExtdByMaster(syncData.getSyncDataId().toArray(new Long[syncData.getSyncDataId().size()]), syncData.getSyncDateUserName());
    }

    /**
     * 主数据修改同步
     * @param syncData
     */
    public void masterUpdateSync(MdmSyncData syncData) {
        BaseResultBean<M> baseResultBean =
                baseMdmBusiModelCloud.whiteFindByMasterIds(syncData.getSyncDataId().toArray(new Long[syncData.getSyncDataId().size()]));
        if (!baseResultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MDM.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail(baseResultBean.getErrorMsg())
                    .build();
        }

        List<M> modelBeanList = baseResultBean.getResultList();
        updateBusiList(transform(modelBeanList));
    }

    /**
     * 主数据删除同步
     * @param syncData
     */
    public void masterDeleteSync(MdmSyncData syncData) {
        baseMdmBusiExtdCloud.whiteDeleteWeaklyByMasterIds(syncData.getSyncDataId().toArray(new Long[syncData.getSyncDataId().size()]),syncData.getSyncDateUserName());
    }

    /**
     * 业务数据新增同步
     * @param syncData
     */
    public void busiInsertSync(MdmSyncData syncData){
        BaseResultBean<M> baseResultBean =
                baseMdmBusiModelCloud.whiteFindByIds(syncData.getSyncDataId().toArray(new Long[syncData.getSyncDataId().size()]));
        if (!baseResultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MDM.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail(baseResultBean.getErrorMsg())
                    .build();
        }

        List<M> modelBeanList = baseResultBean.getResultList();
        insertBusiList(transform(modelBeanList));
    }

    /**
     * 业务数据修改同步
     * @param syncData
     */
    public void busiUpdateSync(MdmSyncData syncData){
        BaseResultBean<M> baseResultBean =
                baseMdmBusiModelCloud.findByIds(syncData.getSyncDataId().toArray(new Long[syncData.getSyncDataId().size()]));
        if (!baseResultBean.isSuccess()) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MDM.getCode())
                    .setErrorCode(ImppExceptionEnum.CLOUD_CALL_EXCEPTION.getCode())
                    .setErrorDetail(baseResultBean.getErrorMsg())
                    .build();
        }

        List<M> modelBeanList = baseResultBean.getResultList();
        updateBusiList(transform(modelBeanList));

    }

    /**
     * 业务数据删除同步
     * @param syncData
     */
    public void busiDeleteSync(MdmSyncData syncData){
        deleteBusiList(syncData.getSyncDataId());
    }

    /**
     * 类型转换
     * @param syncDataList
     * @return
     */
    public abstract List<T> transform(List<M> syncDataList);

    /**
     * 新增业务数据
     * @param beanList
     */
    public abstract void insertBusiList(List<T> beanList);

    /**
     * 修改业务数据
     * @param beanList
     */
    public abstract void updateBusiList(List<T> beanList);

    /**
     * 删除业务数据
     * @param idList
     */
    public abstract void deleteBusiList(List<Long> idList);

}
